import { html } from "uhtml";
import { retrieveOrGenerateKeyPair } from "../utils/crypto.ts";
import { signal } from "uhtml/preactive";

const URL_INPUT_ID = "request-url";
const COPY_BUTTON_ID = "copy-button";
const requestUrl = signal<undefined | URL>(undefined);

export function RequestPage() {
  return html`
    <details open>
      <summary><strong>How To Use:</strong></summary>
      <ol>
        <li>Send the link below to someone you want to get a secret from.</li>
        <li>They add their secret and share the URL the app generates.</li>
        <li>Only your browser can open the URL and decrypt the secret.</li>
      </ol>
    </details>

    <form>
      <input
        type="text"
        name="request-url"
        .id=${URL_INPUT_ID}
        .value=${requestUrl && requestUrl.toString()}
        placeholder=""
        aria-label="Request URL"
        required
      />
      <button
        aria-label="Copy to Clipboard"
        .id=${COPY_BUTTON_ID}
        @click=${(event: Event) =>
          copyToClipboard(event, requestUrl && requestUrl.toString())}
      >
        Copy to Clipboard
      </button>
    </form>
  `;
}

async function copyToClipboard(event: Event, url: string) {
  try {
    event.preventDefault();
    await navigator.clipboard.writeText(url);
    alert("Copied to clipboard!");
  } catch (err) {
    console.error("Failed to copy to clipboard: ", err);
  }
}

export async function generateRequestUrl() {
  try {
    const { publicKey: ecdhPublic } = await retrieveOrGenerateKeyPair();
    const ecdhPublicJwk = await window.crypto.subtle.exportKey(
      "jwk",
      ecdhPublic
    );
    const updatedUrl = new URL(window.location.toString());
    updatedUrl.search = "";
    updatedUrl.hash = `p=${btoa(JSON.stringify(ecdhPublicJwk))}`;
    requestUrl.value = updatedUrl;
  } catch (err) {
    console.error("Failed to generate request URL: ", err);
  }
}
