import { html } from "uhtml";
import {
  deriveSharedSecret,
  encrypt,
  retrieveOrGenerateKeyPair,
} from "../utils/crypto.ts";
import { signal } from "uhtml/preactive";
import { LOCAL_STORAGE_KEYS } from "../utils/store.ts";
import { sendNotification } from "../components/settings/NotificationManager.ts";

const messageToEncrypt = signal("");
const encryptedUrl = signal("");
const isCopyButtonDisabled = signal(true);

export const SendPage = () => {
  return html`
    <details open>
      <summary>How To Use:</summary>
      <ol>
        <li>
          Enter the information you want to send back to the original requester
          into the text input below.
        </li>
        <li>
          Click on 'Generate Response'. This will create a new URL that contains
          your encrypted message.
        </li>
        <li>
          Send the newly generated URL back to the original requester. Only
          their browser will be able to decrypt the message.
        </li>
      </ol>
    </details>
    <form>
      <input
        type="text"
        name="message"
        placeholder="Enter your message here..."
        aria-label="Messag
 to Encrypt"
        required
        value=${messageToEncrypt.value}
        @input=${(e: Event) => {
          messageToEncrypt.value = (e.target as HTMLInputElement).value;
        }}
      />
      <input
        type="submit"
        value="Generate Response"
        aria-label="Generate Response"
        @click=${encryptData}
      />
      <input
        type="text"
        name="encrypted-url"
        placeholder="URL with your encrypted response
 will appear here..."
        aria-label="Encrypted URL"
        value=${encryptedUrl.value}
      />
      <button
        aria-label="Copy to Clipboard"
        @click=${copyToClipboard}
        disabled=${isCopyButtonDisabled.value}
      >
        Copy to Clipboard
      </button>
    </form>
  `;
};

async function copyToClipboard(event: Event) {
  event.preventDefault();
  await navigator.clipboard.writeText(encryptedUrl.value);
  alert(
    "Copied to clipboard! Send this URL to the requester, or they will be notified via a Push Notification if they have it enabled."
  );
}

async function encryptData(event: Event) {
  event.preventDefault();
  const key = localStorage.getItem(LOCAL_STORAGE_KEYS.REQUEST_PUBLIC_KEY)!;

  // Parse the 'p' parameter to get publicA
  const publicAJwk = JSON.parse(atob(key));

  // Import publicA as a CryptoKey
  const publicA = await window.crypto.subtle.importKey(
    "jwk",
    publicAJwk,
    {
      name: "ECDH",
      namedCurve: "P-256",
    },
    true,
    []
  );

  const keyPairB = await retrieveOrGenerateKeyPair();

  // Derive the AES key from your private key and the recipient's public key
  const aesKey = await deriveSharedSecret(keyPairB.privateKey, publicA);

  // Encrypt the message input value using the AES key
  const { encryptedData, iv } = await encrypt(aesKey, messageToEncrypt.value);

  const ecdhPublicJwk = await window.crypto.subtle.exportKey(
    "jwk",
    keyPairB.publicKey
  );
  const url = new URL(window.location.toString());
  url.search = "";
  url.hash = `p=${encodeURIComponent(
    btoa(JSON.stringify(ecdhPublicJwk))
  )}&iv=${encodeURIComponent(
    btoa(String.fromCharCode.apply(null, Array.from(iv)))
  )}&m=${encodeURIComponent(
    btoa(
      String.fromCharCode.apply(null, Array.from(new Uint8Array(encryptedData)))
    )
  )}`;

  sendNotification(publicA, url);

  encryptedUrl.value = url.toString();
  isCopyButtonDisabled.value = false;
}
